from dateutil import parser
from datetime import datetime
from cbutil.util import remove_none

from kadi_apy import KadiManager

import logging
import builtins
import re
from typing import Iterable
logger = logging.getLogger(__file__)


def create_metadatum(key: str, type_name: str, value, **kwargs) -> dict:
    """ Create a dict so that it can be used for kadiapy is add_metadatum."""
    if isinstance(type_name, type):
        type_name = type_name.__name__
    return {"key": key,
            "type": type_name,
            "value": value,
            **kwargs}


def get_type_name(value) -> str:
    if not isinstance(value, str):
        return type(value).__name__
    for possible_type in [int, float]:
        try:
            possible_type(value)
        except ValueError:
            pass
        else:
            return possible_type.__name__

    try:
        parser.parse(value)
    except ValueError:
        logger.debug(f"{value} could not be parse as date")
        pass
    else:
        return "date"

    return str.__name__


def handle_dates(value) -> str:
    try:
        return str(datetime.fromtimestamp(int(value)))
    except (ValueError, OverflowError, OSError):
        pass
    return value


def transform_to_type(type_name, value: str):
    try:
        return getattr(builtins, type_name)(value)
    except AttributeError as e:
        raise ValueError(f"could not transform {value} to {type_name} {e}")


def build_meta_data(tags: dict):
    for key, value in remove_none(tags).items():
        type_name = get_type_name(value)
        if key == "time" or type_name == "date":
            type_name = "date"
            value = handle_dates(value)
        else:
            value = transform_to_type(type_name, value)
        yield create_metadatum(key, type_name, value)


def create_kadi_record(manager: KadiManager, identifier: str, **kwargs):
    return manager.record(identifier=identifier,
                          create=True,
                          **kwargs,
                          )


def get_all_collection_ids(manager: KadiManager, parent_collection_id: int) -> Iterable[int]:
    meta_collection = manager.collection(id=parent_collection_id, create=False)
    page = 1
    while True:
        collections = meta_collection.get_collections(page=page)
        collection_json = collections.json()
        for item in collection_json["items"]:
            yield item["id"]
        if page <= collection_json["_pagination"]["total_pages"]:
            page += 1
        else:
            break


def get_all_record_ids(manager: KadiManager, collection_id: int) -> Iterable[int]:
    meta_collection = manager.collection(id=collection_id, create=False)
    page = 1
    while True:
        collections = meta_collection.get_records(page=page)
        collection_json = collections.json()
        for item in collection_json["items"]:
            yield item["id"]
        if page <= collection_json["_pagination"]["total_pages"]:
            page += 1
        else:
            break


def add_user_to_collection(manager: KadiManager, collection_id: int, user_id: int, *, role="admin"):
    collection = manager.collection(id=collection_id, create=False)
    collection.add_user(user_id=user_id, role_name=role)


def add_user_to_record(manager: KadiManager, record_id: int, user_id: int, *, role="admin"):
    collection = manager.record(id=record_id, create=False)
    collection.add_user(user_id=user_id, role_name=role)


def get_records(manager: KadiManager, record_ids: Iterable[int]):
    for r in record_ids:
        yield manager.record(id=r, create=False)


def filter_record_after_title(records, pattern):
    for r in records:
        if re.match(pattern, r.meta["title"]):
            yield r


def connect_logs(records):
    meta_logs = filter_record_after_title(records, r"fe2ti.*\.o[0-9]+")
    for meta_log in meta_logs:
        name_start = meta_log.meta["title"].split('.')[0]
        for pl in filter_record_after_title(records, f"{name_start}_.*"):
            logger.info(f"Linking {meta_log.id} with {pl.id}")
            logger.debug(f"Linking {meta_log.meta['title']} with {pl.meta['title']}")
            pl.link_record(meta_log.id,  "application call")


def connect_likwid_files(records):
    programm_logs = filter_record_after_title(records, "fe2ti.*_[6-8]_1e-[4,6,8]")
    for p_log in programm_logs:
        likwid_log = list(filter_record_after_title(records, f"likwid_MEM_DP_{p_log.meta['title']}"))
        assert len(likwid_log) == 1, "found more than one likwid log"
        logger.info(f"Linking {p_log.id} with {likwid_log[0].id}")
        logger.info(f"Linking {p_log.meta['title']} with {likwid_log[0].meta['title']}")
        likwid_log[0].link_record(p_log.id, "hardware performance counter")


def get_record_links(record):
    return record.get_record_links()


def remove_records_links_by_name(records, name):
    for record in records:
        links = get_record_links(record).json()
        for link in links['items']:
            if link['name'] == name:
                logger.info(f"Removing link {link['id']}")
                record.delete_record_link(link['id'])
