from typing import List, Dict, Any
from cbutil.postprocessing.plain_text import json2dict
from cbutil.util import get_from_nested_dict
import logging

logger = logging.getLogger(__file__)


def get_stat_sub_dict(data: Dict[str, Any], performance_group: str) -> Dict[str, Any]:
    """
    Given a dictionary of data and the performance group name, returns the sub-dictionary containing the metric STAT.

    Args:
        data (Dict[str, Any]): A dictionary of data from a Likwid JSON file.
        performance_group (str): The name of the performance group to extract the sub-dictionary from.

    Returns:
        Dict[str, Any]: The sub-dictionary containing the metric STAT.
    """

    return get_from_nested_dict(data, [performance_group, performance_group, 'Metric STAT'])


def extract_metric_value(metric_stat: Dict[str, Any], keys: List[str], suffix: str) -> Dict[str, Any]:
    """
    Given the metric STAT sub-dictionary, a list of keys, and a suffix string, returns a dictionary with the key-suffix
    pairs from the metric STAT sub-dictionary.

    Args:
        metric_stat (Dict[str, Any]): The sub-dictionary containing the metric STAT.
        keys (List[str]): The list of keys to extract the key-suffix pairs from.
        suffix (str): The suffix string to append to each key in the returned dictionary.

    Returns:
        Dict[str, Any]: The dictionary containing the key-suffix pairs extracted from the metric STAT sub-dictionary.
    """
    def get_tuple(key):
        try:
            return f'{key} {suffix}', float(get_from_nested_dict(metric_stat, [key, suffix]))
        except (KeyError, ValueError) as e:
            logger.warning(f"Could use {key} because of {e}")
            return None

    return {
        entry[0]: entry[1]
        for key in keys if (entry := get_tuple(key)) is not None
    }


def parse_likwid_json(file: str,
                      performance_group: str,
                      *,
                      sum_keys: List[str],
                      avg_keys: List[str] = [],
                      min_keys: List[str] = [],
                      max_keys: List[str] = []) -> Dict[str, Any]:
    """
    Given the path to a Likwid JSON file, the name of the performance group, and a set of metric keys, returns a
    dictionary containing the extracted metric values.

    Args:
        file (str): The path to the Likwid JSON file.
        performance_group (str): The name of the performance group to extract the metric values from.
        sum_keys (List[str]): The list of metric keys to extract the sum values for.
        avg_keys (List[str], optional): The list of metric keys to extract the average values for. Defaults to [].
        min_keys (List[str], optional): The list of metric keys to extract the minimum values for. Defaults to [].
        max_keys (List[str], optional): The list of metric keys to extract the maximum values for. Defaults to [].

    Returns:
        Dict[str, Any]: A dictionary containing the extracted metric values, with keys of the form "key suffix" (e.g.
        "cycles Sum", "L3 cache hit ratio Avg", etc.).
    """

    parsed_data = {}
    data = json2dict(file)

    metric_stat = get_stat_sub_dict(data, performance_group)
    for keys, suffix in [(sum_keys, 'Sum'), (avg_keys, 'Avg'), (min_keys, 'Min'), (max_keys, 'Max')]:
        parsed_data.update(extract_metric_value(metric_stat, keys, suffix))
    return parsed_data
