import numpy as np
import pandas as pd
import re
from cbutil.ncu_keys import (
    memory_write_data_key,
    memory_read_data_key,
    memory_data_key,
    memory_write_bandwidth_key,
    memory_read_bandwidth_key,
    memory_bandwidth_key,
    runtime_key,
    smsp_cycles_key,
    smsp_cycles_total_key,
    fp_inst_per_cycle_key,
    total_fp_inst_key,
    operational_intensity_key,
    p_max_key,
    dp_key,
)


def parse_ncu_csv(file_name) -> pd.DataFrame:
    return pd.read_csv(file_name, header=[0, 1])


def get_unit(col):
    return col.columns[0]


def detect_prefix(unit):
    if unit[0] == "G":
        return 1e9
    elif unit[0] == "M":
        return 1e6
    elif unit[0] == "K":
        return 1e3
    elif unit[0] == "m":
        return 1e-3
    elif unit[0] == "u":
        return 1e-6
    elif unit[0] == "n":
        return 1e-9
    else:
        return 1


def get_normalized(col):
    unit = get_unit(col)
    if "/" in unit:
        factor = np.divide(*[detect_prefix(u) for u in unit.split("/")])
    else:
        factor = detect_prefix(unit)
    return factor * col


def add_unit_prefix(value, prefix: str):
    return value / detect_prefix(prefix)


def normalize_and_add_prefix(value, prefix: str):
    return add_unit_prefix(get_normalized(value), prefix)


def extract_raw_counter(df: pd.DataFrame):
    fields = pd.DataFrame()
    tags = pd.DataFrame()

    tags["Block Size"] = df["Block Size"]
    tags["Grid Size"] = df["Grid Size"]
    tags["GPU"] = df["device__attribute_display_name"].str.replace(" ", "")

    fields[memory_write_data_key] = normalize_and_add_prefix(
        df["dram__bytes_write.sum"], "G"
    )
    fields[memory_read_data_key] = normalize_and_add_prefix(
        df["dram__bytes_read.sum"], "G"
    )
    fields[memory_data_key] = (
        fields[memory_write_data_key] + fields[memory_read_data_key]
    )

    fields[memory_write_bandwidth_key] = normalize_and_add_prefix(
        df["dram__bytes_write.sum.per_second"], "M"
    )
    fields[memory_read_bandwidth_key] = normalize_and_add_prefix(
        df["dram__bytes_read.sum.per_second"], "M"
    )
    fields[memory_bandwidth_key] = normalize_and_add_prefix(
        df["dram__bytes.sum.per_second"], "M"
    )
    fields[runtime_key] = get_normalized(df["gpu__time_duration.sum"])

    fields[smsp_cycles_key] = get_normalized(
        df["smsp__cycles_elapsed.avg.per_second"])
    fields[smsp_cycles_total_key] = fields[smsp_cycles_key] * fields[runtime_key]
    fields[fp_inst_per_cycle_key] = (
        2 * df["smsp__sass_thread_inst_executed_op_dfma_pred_on.sum.per_cycle_elapsed"]
        + df["smsp__sass_thread_inst_executed_op_dadd_pred_on.sum.per_cycle_elapsed"]
        + df["smsp__sass_thread_inst_executed_op_dmul_pred_on.sum.per_cycle_elapsed"]
    )
    fields[total_fp_inst_key] = (
        fields[fp_inst_per_cycle_key] * fields[smsp_cycles_total_key]
    )

    fields[operational_intensity_key] = fields[total_fp_inst_key] / (
        fields[memory_data_key] * 1e9
    )
    fields[p_max_key] = add_unit_prefix(
        fields[operational_intensity_key] *
        fields[memory_bandwidth_key] * 1e6, "M"
    )
    fields[dp_key] = (
        np.divide(np.asarray(fields[total_fp_inst_key]),
                  fields[runtime_key]) / 1e6
    )
    return fields, tags


def tail_to_dict(df):
    return df.tail(1).iloc[0].to_dict()


def extract_from_csv(file_name: str):
    fields, tags = extract_raw_counter(parse_ncu_csv(file_name))
    return tail_to_dict(fields), tail_to_dict(tags)
