import logging
logger = logging.getLogger(__file__)


def get_tag_keys(client, measurement):
    tag_keys = client.query(f"SHOW TAG KEYS FROM {measurement}")
    return [tag_key['tagKey'] for tag_key in tag_keys.get_points()]


def get_field_keys(client, measurement):
    field_keys = client.query(f"SHOW FIELD KEYS FROM {measurement}")
    return [field_key['fieldKey'] for field_key in field_keys.get_points()]


def get_data_points(client, measurement):
    data = client.query(f"SELECT * FROM {measurement}")
    yield from data.get_points()


def apply_updates(old_data, update_mapping):
    """
    Apply updates to the old data.

    Args:
        old_data: A list of data points in the old format.
        update_mapping: A dictionary of mapping rules to apply.

    Yields:
        A new data point with updates applied.

    Examples:
        >>> old_data = [{'solver': 0, 'value': 1.23}, {'solver': 1, 'value': 4.56}]
        >>> update_mapping = {'solver': {0: 'pardiso', 1: 'mumps'}}
        >>> new_data = list(apply_updates(old_data, update_mapping))
        >>> assert new_data == [{'solver': 'pardiso', 'value': 1.23}, {'solver': 'mumps', 'value': 4.56'}]

    """
    for data_point in old_data:
        for update_key, mapping in update_mapping.items():
            current_value = data_point.get(update_key, None)
            for old_value, new_value in mapping.items():
                if current_value == old_value:
                    data_point[update_key] = new_value

        yield data_point


def restructure_data(client, data, measurement, tag_keys, field_keys):
    for data_point in data:
        tags = {k: v for k, v in data_point.items() if k in tag_keys}
        fields = {k: v for k, v in data_point.items() if k in field_keys}
        yield {
            "measurement": measurement,
            "time": data_point["time"],
            "tags": tags,
            "fields": fields,
        }


def get_updated_data(client, measurement, mapping):
    data = get_data_points(client, measurement)
    data = apply_updates(data, mapping)
    tag_keys = get_tag_keys(client, measurement)
    field_keys = get_field_keys(client, measurement)
    yield from restructure_data(client, data, measurement, tag_keys, field_keys)
