from dataclasses import asdict, dataclass
from typing import List, Union

from grafanalib.core import (
    Annotations,
    Dashboard,
    DashboardList, GridPos,
    Panel, Repeat, RowPanel, Template,
    Templating, Time, TimeSeries
)
from grafanalib.influxdb import InfluxDBTarget

from dashboards.annotations import Annotation
from dashboards.influx_queries import Query
from dashboards.legends import AxisLabel, Units


def get_influx_target(target_query: str, result_format: str = 'time_series', alias: str = "", **kwargs) -> InfluxDBTarget:
    return InfluxDBTarget(
        query=target_query,
        format=result_format,
        alias=alias,
        **{k: v for k, v in kwargs.items() if v is not None},
    )


def get_annotation(*args, **kwargs):
    return Annotations([Annotation(*args, **kwargs).to_json_data()])


def get_commit_annotation(datasource: str, color: str, name: str, measurment_name: str, *, commit_key: str = "commit"):
    query = (f"SELECT first(\"{commit_key}\") FROM "
             f"( SELECT * FROM \"{measurment_name}\" "
             f"WHERE host =~ /^$host$/ AND $timeFilter) GROUP BY \"{commit_key}\"")
    return get_annotation(datasource, color, name, query)


def get_grid_pos(h: int, w: int, x: int, y: int):
    return GridPos(h, w, x, y)


def pack_in_row(title: str, panels: Union[Panel, List[Panel]],
                **kwargs) -> RowPanel:

    if isinstance(panels, Panel):
        panels = [panels]

    return RowPanel(
        title=title,
        collapsed=True,
        panels=panels,
        **kwargs,
    )


def get_dashboard_list(title: str, tags: List[str], **kwargs) -> DashboardList:
    return DashboardList(
        title=title,
        showHeadings=False,
        showSearch=True,
        showStarred=False,
        searchTags=tags,
        **kwargs
    )


@dataclass
class DashboardOptions:
    title: str
    description: str
    tags: List[str]
    timezone: str = "browser"


def build_row_repeat_dashboard(
    options: DashboardOptions,
    row_repeat_var: Template,
    panel_repeat_var: Template,
    dataSource: str,
    measurment_name: str,
    panel_query: Query,
    unit: Units,
    axisLabel: AxisLabel,
    other_vars: List[Template] = None,
    annotations: Annotations = Annotations(),
    alias: str = '',
) -> Dashboard:
    """Build a Dashboard that takes one query and repeats that with 2 variables."""
    time_series_kwargs = {
        'title': f'{panel_repeat_var.name}: ${panel_repeat_var.name}',
        'dataSource': dataSource,
        'targets': [get_influx_target(str(panel_query), alias=alias)],
        'repeat': Repeat('h', panel_repeat_var.name),
        'unit': unit.value,
        'axisLabel': axisLabel.value,
        'pointSize': 9,
    }
    if other_vars is None:
        other_vars = []

    dashboard = Dashboard(
        **asdict(options),
        rows=[
            pack_in_row(
                title=f'{row_repeat_var.name}: ${row_repeat_var.name}',
                panels=[
                    TimeSeries(
                        **time_series_kwargs,
                    ),
                ],
                repeat=Repeat('v', row_repeat_var.name),
            ),
        ],
        templating=Templating([panel_repeat_var, row_repeat_var, *other_vars]),
        time=Time('now-30d', 'now'),
        annotations=annotations,
    )
    return dashboard.auto_panel_ids()


def build_dashboard(options: DashboardOptions,
                    *,
                    templating=None,
                    **kwargs) -> Dashboard:
    if templating is None:
        templating = []
    dashboard = Dashboard(
        **asdict(options),
        **kwargs,
        templating=Templating(templating),
        time=Time('now-14d', 'now'),
        refresh='5m',
    )

    return dashboard.auto_panel_ids()
