from dashboards.dashboard_base import (DashboardOptions,
                                       build_dashboard,
                                       get_commit_annotation,
                                       pack_in_row,
                                       Repeat,)

from dashboards.overrides import get_line_style_regex_override, get_color_regex_override

from dashboards.panels import PanelInfos, get_time_series_panel, get_text_panel, is_regex
from dashboards.variables import get_dashboard_variable, Filter

from dashboards.influx_queries import join_variable_and
from dashboards.legends import Units

from dashboards.influx_queries import Query

description_markdown = r"""

  - Linestyle indicates the compiler:
      - gcc -> dashed lines
      - intel -> full lines
  - Color indicates the solver:
      - pardiso -> green
      - umfpack -> yellow
      - ilu -> blue
  - benchmark setups:
      - fe2ti216: Domain size 2x2x2 solving all microproblems
      - fe2ti1728: Domain size 8x8x1 solving only 216 micro problems
  - List of generated plots can be found [https://www10.cs.fau.de/plots](https://www10.cs.fau.de/plots/)
  - All "raw" output files are uploaded to the [Kadi4Mat](https://kadi4mat.iam-cms.kit.edu/collections/1625)
  - Information about the different Hosts on the NHR@FAU [Testcluster](https://hpc.fau.de/systems-services/documentation-instructions/clusters/test-cluster/)

"""

PARDISO_COLOR = 'rgb(86,166, 75)'
UMFPACK_COLOR = 'rgb(242, 204, 12)'
ILU_COLOR = 'rgb(50, 116, 217)'
ILU_COLOR_MEDIUM = 'rgb(87, 148, 242)'
ILU_COLOR_LIGHT = 'rgb(138, 184, 255)'
INTEL_LINESTYLE = "solid"
GCC_LINESTYLE = "dashed"


def dashboard_fe2ti():
    data_source = "fe2ti"
    measurment_name = "fe2ti"
    row_repeat = "micro_problem_size"
    options = DashboardOptions(
        title="FE2ti Benchmarks",
        description="Benchmarks for fe2ti",
        tags=['benchmark', 'fe2ti'],
        timezone="browser",
    )

    filters = [
        Filter("micro_problem_size", True, '8'),
        Filter("host", False, 'skylakesp2'),
        Filter("branch"),
        Filter("benchmark_name"),
        Filter("compiler"),
        Filter("solver"),
        Filter("mpi_ranks"),
        Filter("omp_threads"),
        Filter("ksp_tol")
    ]

    fields = [PanelInfos("Time to solution", Units.seconds),
              PanelInfos("micro_solve", Units.seconds),
              PanelInfos("KSP_Macro", Units.seconds),
              PanelInfos("DP [MFLOP/s] STAT Sum", Units.mflop_sec),
              PanelInfos("Operational intensity STAT Sum", Units.flop_per_byte),
              PanelInfos('AVX DP [MFLOP/s] STAT Sum" / "DP [MFLOP/s] STAT Sum', Units.percent),
              PanelInfos('Memory bandwidth [MBytes/s] STAT Sum', Units.mbytes_per_second),
              PanelInfos('Memory write bandwidth [MBytes/s] STAT Sum', Units.mbytes_per_second),
              PanelInfos('Memory read bandwidth [MBytes/s] STAT Sum', Units.mbytes_per_second),
              PanelInfos('Memory data volume [GBytes] STAT Sum', Units.gigabyte),
              PanelInfos('Memory read data volume [GBytes] STAT Sum', Units.gigabyte),
              PanelInfos('Memory write data volume [GBytes] STAT Sum', Units.gigabyte),
              PanelInfos('Energy [J] STAT Sum', Units.joule),
              PanelInfos('Power [W] STAT Sum', Units.watt),
              PanelInfos('Clock [MHz] STAT Avg', Units.megahertz, absthreshold=2000),
              PanelInfos('U1_L2_diff", "U2_L2_diff", "U3_L2_diff', Units.number, absthreshold=1e-6),
              ]

    filter_vars = [get_dashboard_variable(filter, measurment_name, data_source) for filter in filters]

    row_repeat_var = [fv for fv in filter_vars if fv.name == row_repeat][0]

    where = join_variable_and([f.name for f in filters])
    annotations = get_commit_annotation(data_source, "red", "commits", measurment_name)

    overrides = [*[
        get_color_regex_override(f".*[Ss]olver.*: {solver}.*", color)
        for solver, color in [("pardiso", PARDISO_COLOR), ("umfpack", UMFPACK_COLOR), ("ilu", ILU_COLOR)]
    ],
        get_color_regex_override("/.*ksp_tol: 1e-4.*.*[Ss]olver.*: ilu.*/", ILU_COLOR_LIGHT),
        get_color_regex_override("/.*ksp_tol: 1e-6.*.*[Ss]olver.*: ilu.*/", ILU_COLOR_MEDIUM),
        get_line_style_regex_override(".*intel.*", INTEL_LINESTYLE),
        get_line_style_regex_override(".*gcc.*", GCC_LINESTYLE)
    ]
    description_panel = pack_in_row("Legend", get_text_panel(description_markdown, title="FE2TI benchmarks"))

    panels = [
        get_time_series_panel(
            field,
            data_source,
            [Query(select_=field.name,
                  from_=measurment_name,
                  where_=where,
                  group_by=[f.name for f in filters],
                  from_string=not is_regex(measurment_name),
                  select_string=not is_regex(field.name))
                  ],
            overrides=overrides,
        )
        for field in fields]

    row = pack_in_row(
        title=f"{row_repeat}: ${row_repeat_var.name}",
        panels=[*panels],
        repeat=Repeat('v', row_repeat_var.name),
    )
    return build_dashboard(options,
                           rows=[description_panel, row],
                           templating=[*filter_vars],
                           annotations=annotations)
