from cbutil.ncu_keys import (
    dp_key,
    memory_bandwidth_key,
    memory_data_key,
    memory_read_bandwidth_key,
    memory_read_data_key,
    memory_write_bandwidth_key,
    memory_write_data_key,
    operational_intensity_key,
    p_max_key,
    runtime_key,
)
from dashboards.dashboard_base import (
    DashboardOptions,
    Repeat,
    build_dashboard,
    get_commit_annotation,
    get_grid_pos,
    pack_in_row,
)
from dashboards.influx_queries import join_variable_and, get_variable_condition_with_tag, get_variable_tag, join_conditions, get_variable_condition, get_variable_condition_without_regex, get_variable_condition_unbounded
from dashboards.legends import Units, AxisLabel
from dashboards.overrides import get_color_regex_override, get_line_style_regex_override
from dashboards.panels import PanelInfos, get_time_series_panel, get_table_panel, get_bar_chart_panel, get_pie_chart_panel, is_regex
from dashboards.variables import (
    Filter,
    get_dashboard_variable,
    get_time_dependend_dashboard_variable,
    get_field_keys_dashboard_variable,
)
from dashboards.influx_queries import Query


def dashboard_uniformgridcpu():
    data_source = "InfluxDB-1"
    arch = "CPU"
    measurment_name = f"UniformGrid{arch}"
    row_repeat = "host"

    options = DashboardOptions(
        title=f"Uniform Grid {arch}",
        description=f"Benchmark dashboard for the Uniform Grid {arch} Benchmark from walberla",
        tags=[arch, "benchmark", "walberla", "Uniform Grid"],
        timezone="browser",
    )

    filters = [
        Filter(row_repeat),
        Filter("project_id", multi=True, default_value="walberla/walberla"),
        Filter("branch", multi=True, default_value="master"),
        Filter("collisionSetup"),
        Filter("mpi_num_processes"),
        Filter("streamingPattern"),
        Filter("timeStepStrategy"),
        Filter("stencil"),
        Filter("blocks_0"),
        Filter("blocks_1"),
        Filter("blocks_2"),
        Filter("cellsPerBlock_0"),
        Filter("cellsPerBlock_1"),
        Filter("cellsPerBlock_2"),
        Filter("periodic_0"),
        Filter("periodic_1"),
        Filter("periodic_2"),
    ]

    fields = [
        PanelInfos("Runtime (RDTSC) [s] STAT Max", Units.seconds),
        PanelInfos("mlupsPerProcess", Units.number),
        PanelInfos("DP [MFLOP/s] STAT Sum", Units.mflop_sec),
        PanelInfos("Operational intensity STAT Sum", Units.flop_per_byte),
        PanelInfos(
            'AVX DP [MFLOP/s] STAT Sum" / "DP [MFLOP/s] STAT Sum', Units.percent
        ),
        PanelInfos(
            "Memory bandwidth [MBytes/s] STAT Sum", Units.mbytes_per_second),
        PanelInfos(
            "Memory write bandwidth [MBytes/s] STAT Sum", Units.mbytes_per_second
        ),
        PanelInfos(
            "Memory read bandwidth [MBytes/s] STAT Sum", Units.mbytes_per_second
        ),
        PanelInfos("Memory data volume [GBytes] STAT Sum", Units.gigabyte),
        PanelInfos(
            "Memory read data volume [GBytes] STAT Sum", Units.gigabyte),
        PanelInfos(
            "Memory write data volume [GBytes] STAT Sum", Units.gigabyte),
        PanelInfos("Energy [J] STAT Sum", Units.joule),
        PanelInfos("Power [W] STAT Sum", Units.watt),
        PanelInfos("Clock [MHz] STAT Avg", Units.megahertz),
    ]

    filter_vars = [
        get_dashboard_variable(filter, measurment_name, data_source)
        for filter in filters[1:]
    ]
    filter_vars = [
        get_time_dependend_dashboard_variable(
            filters[0], measurment_name, data_source, inner_field_key="mlupsPerProcess"
        ),
        *filter_vars,
    ]

    row_repeat_var = [fv for fv in filter_vars if fv.name == row_repeat][0]

    where = join_variable_and([f.name for f in filters])
    annotations = get_commit_annotation(
        data_source, "red", "commits", measurment_name)

    panels = [
        get_time_series_panel(
            field,
            data_source,
            [Query(select_=field.name,
                  from_=measurment_name,
                  where_=where,
                  group_by=[f.name for f in filters],
                  from_string=not is_regex(measurment_name),
                  select_string=not is_regex(field.name))],
        )
        for field in fields
    ]

    row = pack_in_row(
        title=f"{row_repeat}: ${row_repeat_var.name}",
        panels=[*panels],
        repeat=Repeat("v", row_repeat_var.name),
    )
    return build_dashboard(
        options, rows=[row], templating=[*filter_vars], annotations=annotations
    )


def dashboard_uniformgridgpu():
    data_source = "InfluxDB-1"
    arch = "GPU"
    measurment_name = f"UniformGrid{arch}"
    row_repeat = "host"

    options = DashboardOptions(
        title=f"Uniform Grid {arch}",
        description=f"Benchmark dashboard for the Uniform Grid {arch} Benchmark from walberla",
        tags=[arch, "benchmark", "walberla", "Uniform Grid"],
        timezone="browser",
    )

    filters = [
        Filter("host", multi=False, default_value="medusa"),
        Filter("project_id", multi=True, default_value="walberla/walberla"),
        Filter("branch", multi=True, default_value="master"),
        Filter("GPU"),
        Filter("collisionSetup"),
        Filter("mpi_num_processes"),
        Filter("streamingPattern"),
        Filter("timeStepStrategy"),
        Filter("stencil"),
        Filter("blocks_0"),
        Filter("blocks_1"),
        Filter("blocks_2"),
        Filter("cellsPerBlock_0"),
        Filter("cellsPerBlock_1"),
        Filter("cellsPerBlock_2"),
        Filter("gpuBlockSize_0"),
        Filter("gpuBlockSize_1"),
        Filter("gpuBlockSize_2"),
        Filter("periodic_0"),
        Filter("periodic_1"),
        Filter("periodic_2"),
    ]

    fields = [
        PanelInfos("mlupsPerProcess", Units.number),
    ]

    filter_vars = [
        get_dashboard_variable(filter, measurment_name, data_source)
        for filter in filters
    ]

    row_repeat_var = [fv for fv in filter_vars if fv.name == row_repeat][0]

    where = join_variable_and([f.name for f in filters])
    annotations = get_commit_annotation(
        data_source, "red", "commits", measurment_name)

    panels = [
        get_time_series_panel(
            field,
            data_source,
            [Query(select_=field.name,
                  from_=measurment_name,
                  where_=where,
                  group_by=[f.name for f in filters],
                  from_string=not is_regex(measurment_name),
                  select_string=not is_regex(field.name))],
        )
        for field in fields
    ]

    row = pack_in_row(
        title=f"{row_repeat}: ${row_repeat_var.name}",
        panels=[*panels],
        repeat=Repeat("v", row_repeat_var.name),
    )
    return build_dashboard(
        options, rows=[row], templating=[*filter_vars], annotations=annotations
    )


def dashboard_uniformgridgpu_profile():
    data_source = "InfluxDB-1"
    arch = "GPU"
    measurment_name = f"UniformGrid{arch}_profile"
    row_repeat = "host"

    options = DashboardOptions(
        title=f"Uniform Grid {arch} GPU Profiling",
        description=f"Benchmark dashboard for the Uniform Grid {arch} Benchmark from walberla",
        tags=[arch, "benchmark", "walberla", "Uniform Grid", "profiling"],
        timezone="browser",
    )

    filters = [
        Filter("host", multi=False, default_value="medusa"),
        Filter("project_id", multi=True, default_value="walberla/walberla"),
        Filter("branch", multi=True, default_value="master"),
        Filter("GPU"),
        Filter("collisionSetup"),
        Filter("mpi_num_processes"),
        Filter("streamingPattern"),
        Filter("timeStepStrategy"),
        Filter("stencil"),
        Filter("blocks_0"),
        Filter("blocks_1"),
        Filter("blocks_2"),
        Filter("cellsPerBlock_0"),
        Filter("cellsPerBlock_1"),
        Filter("cellsPerBlock_2"),
        Filter("gpuBlockSize_0"),
        Filter("gpuBlockSize_1"),
        Filter("gpuBlockSize_2"),
        Filter("periodic_0"),
        Filter("periodic_1"),
        Filter("periodic_2"),
    ]

    fields = [
        PanelInfos(runtime_key, Units.seconds),
        PanelInfos(dp_key, Units.mflop_sec),
        PanelInfos(p_max_key, Units.mflop_sec),
        PanelInfos(f'{dp_key}"/"{p_max_key}', Units.percent),
        PanelInfos(operational_intensity_key, Units.flop_per_byte),
        PanelInfos(memory_bandwidth_key, Units.mbytes_per_second),
        PanelInfos(memory_write_bandwidth_key, Units.mbytes_per_second),
        PanelInfos(memory_read_bandwidth_key, Units.mbytes_per_second),
        PanelInfos(memory_data_key, Units.gigabyte),
        PanelInfos(memory_write_data_key, Units.gigabyte),
        PanelInfos(memory_read_data_key, Units.gigabyte),
    ]

    filter_vars = [
        get_dashboard_variable(filter, measurment_name, data_source)
        for filter in filters
    ]

    row_repeat_var = [fv for fv in filter_vars if fv.name == row_repeat][0]

    where = join_variable_and([f.name for f in filters])
    annotations = get_commit_annotation(
        data_source, "red", "commits", measurment_name)

    panels = [
        get_time_series_panel(
            field,
            data_source,
            [Query(select_=field.name,
                  from_=measurment_name,
                  where_=where,
                  group_by=[f.name for f in filters],
                  from_string=not is_regex(measurment_name),
                  select_string=not is_regex(field.name))],
        )
        for field in fields
    ]

    row = pack_in_row(
        title=f"{row_repeat}: ${row_repeat_var.name}",
        panels=[*panels],
        repeat=Repeat("v", row_repeat_var.name),
    )
    return build_dashboard(
        options, rows=[row], templating=[*filter_vars], annotations=annotations
    )



def dashboard_uniformgridcpu_relativeperformance():
    data_source = "InfluxDB-1"
    arch = "CPU"
    measurment_name = f"UniformGrid{arch}"
    measurment_name2 = f"roofline"
    row_repeat = "host"

    options = DashboardOptions(
        title=f"Uniform Grid {arch} - Relative Performance",
        description=f"Relative performance benchmark dashboard for the Uniform Grid {arch} Benchmark from walberla",
        tags=[arch, "walberla"],
        timezone="browser",
    )

    filters = [
        Filter("host", multi=False, default_value="icx36"),
        Filter("cellsPerBlock_0", multi=False, default_value="128"),
        Filter("collisionSetup"),
        Filter("project_id", multi=False, default_value="walberla/walberla"),
        Filter("branch", multi=False, default_value="master"),
        Filter("BandwidthBenchmark", multi=False, default_value="bw_stream"),
    ]

    fields = [
        PanelInfos("Relative Performance", Units.number, absthreshold=80),
        PanelInfos("Bandwidth", Units.number, absthreshold=80),
        PanelInfos("Relative Performance", Units.percent, absthreshold=80),
    ]

    filter_vars = [
        get_dashboard_variable(filter, measurment_name, data_source)
        for filter in filters[1:5]
    ]
    filter_vars = [
        get_dashboard_variable(filters[0], measurment_name2, data_source),
        *filter_vars,
        get_field_keys_dashboard_variable(
            filters[5], measurment_name2, data_source
        ),
    ]
       
    row_repeat_var = [fv for fv in filter_vars if fv.name == row_repeat][0]

    where0A2 = join_conditions(
        [
            get_variable_condition_with_tag(filters[i].name) for i in range(2)
        ] + [
            get_variable_condition_without_regex(filters[i].name) for i in [2]
        ] + [
            get_variable_condition(filters[3].name)
        ] + [
            get_variable_condition_without_regex(filters[i].name) for i in [4]
        ], "AND"
        , include_time_filter=True
    )
    where0B2 = get_variable_condition_with_tag(filters[0].name)
    where1A = get_variable_condition_with_tag(filters[0].name)
    where2A2 = join_conditions(
        [
            get_variable_condition_with_tag(filters[i].name) for i in range(2)
        ] + [
            get_variable_condition_without_regex(filters[i].name) for i in [2]
        ] + [
            get_variable_condition(filters[3].name)
        ] + [
            get_variable_condition_without_regex(filters[i].name) for i in [4]
        ], "AND"
        , include_time_filter=True
    )
    where2B2 = get_variable_condition_with_tag(filters[0].name)

    group_by0A2 = ["host", "project_id", "branch", "collisionSetup", "mpi_num_processes", "streamingPattern", "timeStepStrategy", "stencil", "blocks_0", "blocks_1", "blocks_2", "cellsPerBlock_0", "cellsPerBlock_1", "cellsPerBlock_2", "periodic_0", "periodic_1", "periodic_2"]
    group_by0B2 = [get_variable_tag(filters[0].name)]
    group_by1A = [filters[0].name]
    group_by2A2 = ["host", "project_id", "branch", "collisionSetup", "mpi_num_processes", "streamingPattern", "timeStepStrategy", "stencil", "blocks_0", "blocks_1", "blocks_2", "cellsPerBlock_0", "cellsPerBlock_1", "cellsPerBlock_2", "periodic_0", "periodic_1", "periodic_2"]
    group_by2B2 = [get_variable_tag(filters[0].name)]
    
    annotations = get_commit_annotation(
        data_source, "red", "commits", measurment_name2)

    selected_columns0A1 = ", ".join([
        '"host"',
        '"collisionSetup"',
        '"MLUPSperProcess"',
        '"mpi_num_processes"'])
    selected_columns0A2 = ", ".join([
        '(last("mlupsPerProcess")) as "MLUPSperProcess"',
        '"mpi_num_processes"'])
    selected_columns0B1 = ", ".join([
        '"host"',
        '"P_max"'])
    selected_columns0B2 = ", ".join([
        'last($BandwidthBenchmark)/(27*2*8) as "P_max"'])
    selected_columns1A = ", ".join([
        'last($BandwidthBenchmark) as "BW[MByte/s]"',
        'last($BandwidthBenchmark)/(27*2*8) as "P_max[MLUPS]"'])
    selected_columns2A1 = ", ".join([
        '"host"',
        '"collisionSetup"',
        '"MLUPSperProcess"',
        '"mpi_num_processes"'])
    selected_columns2A2 = ", ".join([
        '(last("mlupsPerProcess")) as "MLUPSperProcess"',
        '"mpi_num_processes"'])
    selected_columns2B1 = ", ".join([
        '"host"',
        '"P_max"'])
    selected_columns2B2 = ", ".join([
        'last($BandwidthBenchmark)/(27*2*8) as "P_max"'])


    query0A2 = Query(select_=selected_columns0A2,
                  from_=measurment_name,
                  where_=where0A2,
                  group_by=group_by0A2,
                  from_string=not is_regex(measurment_name),
                  select_string=False)
    query0A1 = Query(selected_columns0A1,
                  from_=query0A2,
                  from_string=not is_regex(measurment_name),
                  select_string=False)
    query0B2 = Query(select_=selected_columns0B2,
                  from_=measurment_name2,
                  where_=where0B2,
                  group_by=group_by0B2,
                  from_string=not is_regex(measurment_name),
                  select_string=False)   
    query0B1 = Query(select_=selected_columns0B1,
                  from_=query0B2,
                  from_string=not is_regex(measurment_name),
                  select_string=False)     

    query1A = Query(select_=selected_columns1A,
                  from_=measurment_name2,
                  where_=where1A,
                  group_by=group_by1A,
                  from_string=not is_regex(measurment_name),
                  select_string=False)

    query2A2 = Query(select_=selected_columns2A2,
                  from_=measurment_name,
                  where_=where2A2,
                  group_by=group_by2A2,
                  from_string=not is_regex(measurment_name),
                  select_string=False)
    query2A1 = Query(selected_columns2A1,
                  from_=query2A2,
                  from_string=not is_regex(measurment_name),
                  select_string=False)
    query2B2 = Query(select_=selected_columns2B2,
                  from_=measurment_name2,
                  where_=where2B2,
                  group_by=group_by2B2,
                  from_string=not is_regex(measurment_name),
                  select_string=False)   
    query2B1 = Query(select_=selected_columns2B1,
                  from_=query2B2,
                  from_string=not is_regex(measurment_name),
                  select_string=False)  

    alias0A = "$tag_collisionSetup"
    alias2A = "$tag_collisionSetup"
                  

    transformations0 = [
    {
      "id": "joinByField",
      "options": {
        "byField": "host",
        "mode": "outerTabular"
      }
    },
    {
      "id": "calculateField",
      "options": {
        "alias": "P",
        "binary": {
          "left": "MLUPSperProcess",
          "operator": "*",
          "right": "mpi_num_processes"
        },
        "mode": "binary",
        "reduce": {
          "reducer": "sum"
        },
        "replaceFields": False
      }
    },
    {
      "id": "calculateField",
      "options": {
        "binary": {
          "left": "P",
          "operator": "/",
          "right": "P_max"
        },
        "mode": "binary",
        "reduce": {
          "reducer": "sum"
        }
      }
    },
    {
      "id": "filterFieldsByName",
      "options": {
        "include": {
          "names": [
            "Time 1",
            "collisionSetup",
            "P",
            "P / P_max",
            "MLUPSperProcess"
          ]
        }
      }
    }
  ]

    transformations2 = [
    {
      "id": "joinByField",
      "options": {
        "byField": "host",
        "mode": "outerTabular"
      }
    },
    {
      "id": "calculateField",
      "options": {
        "alias": "P",
        "binary": {
          "left": "MLUPSperProcess",
          "operator": "*",
          "right": "mpi_num_processes"
        },
        "mode": "binary",
        "reduce": {
          "reducer": "sum"
        },
        "replaceFields": False
      }
    },
    {
      "id": "calculateField",
      "options": {
        "binary": {
          "left": "P",
          "operator": "/",
          "right": "P_max"
        },
        "mode": "binary",
        "reduce": {
          "reducer": "sum"
        }
      }
    },
    {
      "id": "filterFieldsByName",
      "options": {
        "include": {
          "names": [
            "collisionSetup",
            "P / P_max"
          ]
        }
      }
    }
  ]

    panels = [
        # Panel 0
        get_table_panel(
            fields[0],
            data_source,
            [query0A1, query0B1],
            alias_list = [alias0A, ""],
            transformations=transformations0,
            gridPos=get_grid_pos(14, 10, 0, 1),
        ),
        # Panel 1
        get_table_panel(
            fields[1],
            data_source,
            [query1A],
            gridPos=get_grid_pos(5, 10, 10, 1),
        ),
        # Panel 2
        get_bar_chart_panel(
            fields[2],
            data_source,
            [query2A1, query2B1],
            alias_list = [alias2A, ""],
            transformations=transformations2,
            gridPos=get_grid_pos(9, 10, 10, 6),
        ),
    ]

    return build_dashboard(
        options, panels=panels, templating=filter_vars, annotations=annotations
    )



def dashboard_fslbmgravitywave():
    data_source = "InfluxDB-1"
    arch = "CPU"
    measurment_name = "FSLBM_gravitywave"

    options = DashboardOptions(
        title="Free Surface LBM - Gravity Wave",
        description="Benchmark dashboard for the FSBLM Gravitywave Benchmark from walberla",
        tags=[arch, "benchmark", "walberla", "FSLBM"],
        timezone="browser",
    )

    filters = [
        Filter("host"),
        Filter("project_id", multi=True, default_value="walberla/walberla"),
        Filter("branch", multi=True, default_value="master"),
        Filter("numMPIProcesses"),
        Filter("barrierAfterSweep"),
        Filter("blockDecomposition"),
    ]

    fields = [
        PanelInfos("simulationTime", Units.seconds),
        PanelInfos("/.*_percentage/", Units.percent),
        PanelInfos("/Communication:.*_percentage/", Units.percent),
        PanelInfos(r"/Sweep:.*_percentage/", Units.percent),
        PanelInfos(r"/MPI Barrier:.*_percentage/", Units.percent),
    ]

    filter_vars = [
        get_dashboard_variable(filter, measurment_name, data_source)
        for filter in filters
    ]

    where = join_variable_and([f.name for f in filters])
    annotations = get_commit_annotation(
        data_source, "red", "commits", measurment_name)
    groups = ["Communication", "Sweep", "MPI Barrier"]
    colors = ["red", "green", "blue"]
    overrides = [
        get_color_regex_override(f".*{group}:.*", color, mode="fixed")
        for group, color in zip(groups, colors)
    ]

    queries = [
        Query(
            select_=field.name,
            from_=measurment_name,
            where_=where,
            group_by=[f.name for f in filters],
            from_string=not is_regex(measurment_name),
            select_string=not is_regex(field.name)
        )
        for index, field in enumerate(fields)
    ]

    panels = [
        get_time_series_panel(
            fields[0],
            data_source,
            [queries[0]],
            gridPos=get_grid_pos(12, 24, 0, 0),
        )
    ]
    panels.append(
        get_time_series_panel(
            fields[1],
            data_source,
            [queries[1]],
            gridPos=get_grid_pos(12, 12, 0, 13),
            drawStyle="bars",
            stacking={"mode": "normal"},
            fillOpacity=70,
            gradientMode="Opacity",
            tooltipMode="single",
            overrides=overrides,
            colorMode="fixed",
        )
    )

    panels.append(
        get_time_series_panel(
            fields[2],
            data_source,
            [queries[2]],
            gridPos=get_grid_pos(12, 12, 12, 13),
            drawStyle="bars",
            stacking={"mode": "normal"},
            fillOpacity=70,
            gradientMode="Opacity",
            tooltipMode="single",
        )
    )

    panels.append(
        get_time_series_panel(
            fields[3],
            data_source,
            [queries[3]],
            gridPos=get_grid_pos(12, 12, 0, 2 * 13),
            drawStyle="bars",
            stacking={"mode": "normal"},
            fillOpacity=70,
            gradientMode="Opacity",
            tooltipMode="single",
        )
    )

    panels.append(
        get_time_series_panel(
            fields[4],
            data_source,
            [queries[4]],
            gridPos=get_grid_pos(12, 12, 12, 2 * 13),
            drawStyle="bars",
            stacking={"mode": "normal"},
            fillOpacity=70,
            gradientMode="Opacity",
            tooltipMode="single",
        )
    )

    return build_dashboard(
        options, panels=panels, templating=filter_vars, annotations=annotations
    )



def dashboard_percolationgpu():
    data_source = "InfluxDB-1"
    arch = "GPU"
    measurment_name = f"Percolation{arch}"
    measurment_name2 = f"Percolation{arch}_profile"
    row_repeat = "GPU"

    options = DashboardOptions(
        title="Percolation GPU",
        description=f"Benchmark dashboard for the Percolation {arch} Benchmark from walberla",
        tags=[arch, "walberla"],
        timezone="browser",
    )

    filters = [
        Filter("host", multi=False, refresh=2),
        Filter(row_repeat),
        Filter("useParticles"),
    ]

    fields = [
        PanelInfos("MLUPs", Units.mlups, absthreshold=80),
        PanelInfos("Time Steps per Second", absthreshold=80),
        PanelInfos("Performance Flops", Units.mflop_sec, absthreshold=80),
        PanelInfos("Memory Bandwidth", Units.mbytes_per_second, absthreshold=80),
        PanelInfos(r"/.*_average/ useParticles=$useParticles", Units.seconds),
        PanelInfos(r"/.*_average/ useParticles=$useParticles", Units.seconds),
    ]

    whereVariable = join_conditions([get_variable_condition_unbounded(filters[0].name)], "AND",
        include_time_filter=True)
    groupVariable = [filters[0].name]

    filter_vars = [
        get_dashboard_variable(filter, measurment_name, data_source)
        for filter in filters[2:]
    ]
    filter_vars = [
        get_time_dependend_dashboard_variable(
            filters[0], measurment_name, data_source, inner_field_key="MLUPS"
        ),
        get_time_dependend_dashboard_variable(
            filters[1], measurment_name, data_source, inner_field_key="MLUPS", where=whereVariable, group_by=groupVariable
        ),
        *filter_vars
    ]


    annotations = get_commit_annotation(
        data_source, "red", "commits", measurment_name)

    overrides0 = [
        get_line_style_regex_override(f"/.*useParticles: 0/", style="dash")
    ]
        
    groups4 = ["LBM sweep", "Boundary", "Communication"]
    colors4 = ["blue", "semi-dark-yellow", "semi-dark-purple"]
    overrides4 = [
        get_color_regex_override(f"/{group}.*/", color, mode="fixed")
        for group, color in zip(groups4, colors4)
    ]


    where = join_conditions([get_variable_condition_with_tag(filters[i].name) for i in range(3)], "AND",
        include_time_filter=True)

    group_by_elements01 = ["GPU", "branch", "fluidCells", "host", "numParticles", "useParticles", "communicationHidingXWidth", "communicationHidingYWidth", "communicationHidingZWidth", "cores", "numXCellsPerBlock", "numYBlocks", "particleDiameter", "sendDirectlyFromGPU"]
    group_by01 = [get_variable_tag(i) for i in group_by_elements01]
    group_by23 = [get_variable_tag(i.name) for i in filters]
    group_by4 = [get_variable_tag(i) for i in group_by_elements01]


    selected_columns0 = ", ".join([
        '"MLUPS"'])
    selected_columns1 = ", ".join([
        '"timeStepsPerSecond"'])
    selected_columns2 = ", ".join([
        '"DP [MFlop/s]"'])
    selected_columns3 = ", ".join([
        '"Memory bandwidth [MByte/s]"'])
    selected_columns4 = ", ".join([
        '/.*_average/'])
    

    query0 = Query(select_=selected_columns0,
                  from_=measurment_name,
                  where_=where,
                  group_by=group_by01,
                  from_string=not is_regex(measurment_name),
                  select_string=False)
    query1 = Query(select_=selected_columns1,
                  from_=measurment_name,
                  where_=where,
                  group_by=group_by01,
                  from_string=not is_regex(measurment_name),
                  select_string=False)
    query2 = Query(select_=selected_columns2,
                  from_=measurment_name2,
                  where_=where,
                  group_by=group_by23,
                  from_string=not is_regex(measurment_name2),
                  select_string=False)  
    query3 = Query(select_=selected_columns3,
                  from_=measurment_name2,
                  where_=where,
                  group_by=group_by23,
                  from_string=not is_regex(measurment_name2),
                  select_string=False)
    query4 = Query(select_=selected_columns4,
                  from_=measurment_name,
                  where_=where,
                  group_by=group_by4,
                  from_string=not is_regex(measurment_name),
                  select_string=False)


    panels = [
        # Panel 0
        get_time_series_panel(
            fields[0],
            data_source,
            [query0],
            pointSize=5,
            thresholdsStyleMode='off',
            gridPos=get_grid_pos(13, 9, 0, 0),
            overrides=overrides0,
        ),
        # Panel 1
        get_time_series_panel(
            fields[1],
            data_source,
            [query1],
            pointSize=5,
            thresholdsStyleMode='off',
            gridPos=get_grid_pos(13, 8, 9, 0),
        ),
        # Panel 2
        get_time_series_panel(
            fields[2],
            data_source,
            [query2],
            pointSize=5,
            thresholdsStyleMode='off',
            gridPos=get_grid_pos(12, 9, 0, 14),
        ),
        # Panel 3
        get_time_series_panel(
            fields[3],
            data_source,
            [query3],
            pointSize=5,
            thresholdsStyleMode='off',
            gridPos=get_grid_pos(12, 9, 9, 14),
        ),
        # Panel 4
        get_pie_chart_panel(
            fields[4],
            data_source,
            [query4],
            gridPos=get_grid_pos(13, 12, 0, 27),
            repeat=Repeat("h", filters[2].name),
            overrides=overrides4,
        ),
    ]


    # Rows
    row_repeat_var = [fv for fv in filter_vars if fv.name == row_repeat][0]

    row0 = pack_in_row(
        title=f"MLUPS and Time Steps per Second",
        panels=panels[0:2],
    )

    row1 = pack_in_row(
        title="Profiling",
        panels=panels[2:4],
    )
    
    row2 = pack_in_row(
        title=f"Distribution ${row_repeat_var.name}",
        panels=panels[4:5],
        repeat=Repeat("v", row_repeat_var.name),
    )

    rows = [row0, row1, row2]

    return build_dashboard(
        options, rows=rows, templating=filter_vars, annotations=annotations
    )
