import plotly.graph_objs as go
import json

try:
    import importlib.resources as pkg_resources
except ImportError:
    # Try backported to PY<37 `importlib_resources`.
    import importlib_resources as pkg_resources

DEFAULT_MARKER = dict(size=10, color='green')


def get_data(file_name="roofline_data.json"):
    with pkg_resources.open_text(__package__, file_name) as file:
        data = json.load(file)
    return data


def add_compute_roofline(fig, peak_perf: float, ridge_point: float, max_x: float, *, name='Peak Performance'):
    text = f'{name}: {peak_perf/1e12:.4f} TFlop/s'
    fig.add_trace(
        go.Scatter(x=[ridge_point, max_x],
                   y=[peak_perf, peak_perf],
                   mode='lines+text',
                   name=text,
                   text=['', text],
                   textposition="top left",
                   line=dict(color='black', width=4))
    )
    return fig


def add_memory_roofline(fig, peak_perf: float, ridge_point: float, max_bw: float, name=''):
    fig.add_trace(
        go.Scatter(x=[0, ridge_point],
                   y=[0, peak_perf],
                   mode='lines+text',
                   name=f"Memory bandwidth ({name}): {max_bw/1e9:.4f} GB/s",
                   textposition="bottom left",
                   line=dict(color='black', width=4))
    )
    return fig


def add_rooflines(fig, peak_perf, max_bw, max_x, name_perf='Peak Performance', name_bw=''):
    # Calculate oi where oi*max_bw == peak_perf
    ridge_point = peak_perf / max_bw
    fig = add_compute_roofline(fig, peak_perf, ridge_point, max_x, name=name_perf)
    fig = add_memory_roofline(fig, peak_perf, ridge_point, max_bw, name=name_bw)

    return fig


def add_data_point(fig, x, y, *,
                   name="",
                   marker=DEFAULT_MARKER,
                   ):
    fig.add_trace(go.Scatter(
        x=[x],
        y=[y],
        mode='markers',
        marker=marker,
        name=name,
        text=name,
    ))
    return fig


def set_axis(fig):
    # Add roofline axes labels
    common_config = dict(
        type='log',
        zeroline=True,
        exponentformat="SI"
    )
    fig.update_layout(
        xaxis={
            'title': 'Operational Intensity (Flop/Byte)',
            **common_config,
        },
        yaxis={
            'title': 'Performance (Flop/s)',
            **common_config,
        },
    )
    return fig


def create_figure(title: str):
    fig = go.Figure()
    fig = set_axis(fig)
    fig.update_layout(title=title)
    return fig


def create_plot_by_host(hostname: str, title: str, *, max_x=50):
    data = get_data()[hostname]
    max_band = data["bw"]["load"] * 1e6
    perfs = sorted(data["perf"].items(), key=lambda x: x[1], reverse=True)
    fig = create_figure(title)
    for idx, (name, perf) in enumerate(perfs):
        perf_in_flops = perf * 1e6
        add_compute_roofline(fig, perf_in_flops, (perf_in_flops/max_band), max_x, name=name)
        if idx == 0:
            add_memory_roofline(fig, perf_in_flops, perf_in_flops/max_band, max_band, "load")

    return fig


def create_plot(peak_perf, max_band, title):
    fig = create_figure(title)
    fig = add_rooflines(fig, peak_perf, max_band, 100)
    return fig


def add_data_points(fig, data_points, marker=DEFAULT_MARKER):
    for oi, perf, name in data_points:
        fig = add_data_point(fig, oi, perf, name=name, marker=marker)
    return fig
