import os
import pytest
import tempfile
from cbutil.compare_vtu import compare_vtu_files


@pytest.fixture
def setup_test_files():
    # Create a reference VTU file and a result VTU file for testing

    # Assuming the reference VTU file contains data arrays 'U1', 'U2', and 'U3'
    # with values [1.0, 2.0, 3.0], [4.0, 5.0, 6.0], and [7.0, 8.0, 9.0], respectively.
    # The result VTU file contains identical data arrays.

    # Create the reference VTU file
    with tempfile.TemporaryDirectory() as tmp_dir:
        ref_file = f"{tmp_dir}/reference_test_file.vtu"
        result_file = f"{tmp_dir}/result_test_file.vtu"
        with open(ref_file, "w") as f:
            f.write('<?xml version="1.0"?>\n<VTKFile type="UnstructuredGrid" version="0.1" byte_order="LittleEndian">\n')
            f.write('<UnstructuredGrid>\n<PointData>\n')
            for array_name, values in zip(['U1', 'U2', 'U3'], [[1.0, 2.0, 3.0], [4.0, 5.0, 6.0], [7.0, 8.0, 9.0]]):
                f.write(f'<DataArray type="Float64" Name="{array_name}" format="ascii">\n')
                f.write(" ".join(map(str, values)) + "\n")
                f.write("</DataArray>\n")
            f.write('</PointData>\n</UnstructuredGrid>\n</VTKFile>\n')

        # Create the result VTU file (identical to the reference)
        with open(result_file, "w") as f:
            f.write('<?xml version="1.0"?>\n<VTKFile type="UnstructuredGrid" version="0.1" byte_order="LittleEndian">\n')
            f.write('<UnstructuredGrid>\n<PointData>\n')
            for array_name, values in zip(['U1', 'U2', 'U3'], [[1.0, 2.0, 3.0], [4.0, 5.0, 6.0], [7.0, 8.0, 9.0]]):
                f.write(f'<DataArray type="Float64" Name="{array_name}" format="ascii">\n')
                f.write(" ".join(map(str, values)) + "\n")
                f.write("</DataArray>\n")
            f.write('</PointData>\n</UnstructuredGrid>\n</VTKFile>\n')
        yield ref_file, result_file


def test_valid_comparison(setup_test_files):
    ref_file, result_file = setup_test_files
    result = compare_vtu_files(ref_file, result_file, ['U1', 'U2', 'U3'])
    assert result['U1_L2_diff'] == pytest.approx(0.0)
    assert result['U2_L2_diff'] == pytest.approx(0.0)
    assert result['U3_L2_diff'] == pytest.approx(0.0)


def test_missing_reference_file():
    with pytest.raises(FileNotFoundError):
        compare_vtu_files("non_existent_file.vtu", "other_file.vtu", ["U1"])


def test_missing_data_array(setup_test_files):
    ref_file, result_file = setup_test_files
    result = compare_vtu_files(ref_file, result_file, ['U1', 'U2', 'U3', 'U4'])
    assert 'U4_L2_diff' not in result
