import os
import time
from unittest.mock import MagicMock

import pytest

from cbutil.data_points import DataPoint
from cbutil.upload import DBConfig, Uploader, load_config_from_env


@pytest.fixture
def dummy_conf():
    return DBConfig("host", 1234, "user_name", "database", "write_user_pw")


@pytest.fixture
def mock_upload(dummy_conf):
    up = Uploader(dummy_conf)
    up.client = MagicMock(return_value=True)
    return up


@pytest.fixture
def dummy_dp():
    return DataPoint(measurement="Test", time=int(time.time()), fields=dict(), tags=dict())


def setup_env():
    os.environ["INFLUXDB_HOST"] = "INFLUXDB_HOST"
    os.environ["INFLUXDB_PORT"] = "1234"
    os.environ["INFLUXDB_USER_NAME"] = "INFLUXDB_USER_NAME"
    os.environ["INFLUXDB_DATABASE"] = "INFLUXDB_DATABASE"
    os.environ["INFLUXDB_WRITE_USER_PASSWORD"] = "INFLUXDB_WRITE_USER_PASSWORD"


def test_init_with_conf(dummy_conf):
    Uploader(dummy_conf)


def test_load_conf():
    setup_env()

    conf = load_config_from_env()
    assert conf.host == os.environ["INFLUXDB_HOST"]
    assert conf.port == os.environ["INFLUXDB_PORT"]
    assert conf.user_name == os.environ["INFLUXDB_USER_NAME"]
    assert conf.database == os.environ["INFLUXDB_DATABASE"]
    assert conf.write_user_pw == os.environ["INFLUXDB_WRITE_USER_PASSWORD"]

    Uploader(conf)


def test_init_from_env():
    setup_env()
    up = Uploader()
    assert up.config.host == os.environ["INFLUXDB_HOST"]
    assert up.config.port == os.environ["INFLUXDB_PORT"]
    assert up.config.user_name == os.environ["INFLUXDB_USER_NAME"]
    assert up.config.database == os.environ["INFLUXDB_DATABASE"]
    assert up.config.write_user_pw == os.environ["INFLUXDB_WRITE_USER_PASSWORD"]


def test_dry_run(mock_upload):
    dp = DataPoint(measurement="Test", time=int(time.time()), fields=dict(), tags=dict())
    assert mock_upload.upload([dp], dry_run=True, time_precision='s')
    mock_upload.client.write_points.assert_not_called()


def test_upload_as_dp(mock_upload, dummy_dp):
    assert mock_upload.upload([dummy_dp], time_precision='s')
    mock_upload.client.write_points.assert_called_with([dummy_dp.asdict()], time_precision='s')


def test_upload_as_dict(mock_upload, dummy_dp):
    assert mock_upload.upload([dummy_dp.asdict()], time_precision='s')
    mock_upload.client.write_points.assert_called_with([dummy_dp.asdict()], time_precision='s')
